# Внешние обработки для запуска в режиме 1С:Предприятие

В текущем каталоге располагаются внешние обработки для запуска в режиме 1С:Предприятие.

В том числе есть несколько обработок, позволяющих упростить развертывание/тестирование для конфигураций, основанных на БСП.

+ Основной пример (см. ниже пример вызова) - это передача через параметры `/C` команды `"ЗапуститьОбновлениеИнформационнойБазы;ЗавершитьРаботуСистемы"` и одновременная передача через `/Execute "ЗакрытьПредприятие.epf"`.

  + При запуске с такими ключами подключается обработчик ожидания, который проверяет наличие формы с заголовком обновления и при окончании обновления завершает 1С:Предприятие. Данное действие необходимо для полного обновления информационной базы 1С:Предприятия, пока действует блокировка на фоновые задачи и запуск пользователей.

  + также выполняется отключение запроса при завершении работы программы для БСП-конфигураций

  + код запуска

```bat
    @call vrunner run --command "ЗапуститьОбновлениеИнформационнойБазы;ЗавершитьРаботуСистемы;" --execute $runnerRoot\epf\ЗакрытьПредприятие.epf
```

+ **ЗагрузитьРасширение** позволяет подключать расширение в режиме предприятия и получать результат ошибки. Предназначено для подключения в конфигурациях, основанных на БСП. В параметрах /C передается путь к расширению и путь к файлу лога подключения.

+ **ЗагрузитьВнешниеОбработки** позволяет загрузить все внешние обработки и подключить в справочник "Дополнительные отчеты и обработки", т.к. их очень много то первым параметром идет каталог, вторым параметром путь к файлу лога. Все обработки обновляются согласно версиям.

+ **БлокировкаРаботыСВнешнимиРесурсами** позволяет блокировать, разблокировать работу с внешними ресурсами и выключать отдельные регламентные задания (по наименованию).
  
  + выполняется только на конфигурациях со встроенной БСП

  + доступны параметры `ЗапретитьРаботуСВнешнимиРесурсами`, `РазрешитьРаботуСВнешнимиРесурсами` или `ОтключитьРегламентноеЗадание`

  + При использовании параметра `ОтключитьРегламентноеЗадание` требуется через разделитель указать наименование отключаемого регламентного задания, в примере указано задание `ИзвлечениеТекста`

  + код запуска

```bat
      @call vrunner run --command "ОтключитьРегламентноеЗадание;ИзвлечениеТекста;ЗавершитьРаботуСистемы" --execute $runnerRoot\epf\БлокировкаРаботыСВнешнимиРесурсами.epf
```

```bat
      @call vrunner run --command "ЗапретитьРаботуСВнешнимиРесурсами;ЗавершитьРаботуСистемы" --execute $runnerRoot\epf\БлокировкаРаботыСВнешнимиРесурсами.epf
```

+ **СозданиеПользователей** позволяет создать первого пользователя-администратора, если в ИБ еще не существует пользователей. Администратор назначается роль `ПолныеПрава`, если она существует в ИБ.

  + также выполняется отключение запроса при завершении работы программы для БСП-конфигураций

  + код запуска для создания пользователя с именем `Администратор`

```bat
      @call vrunner run --command "СоздатьАдминистратора;Имя=Администратор;ЗавершитьРаботуСистемы" --execute $runnerRoot\epf\СоздатьПользователей.epf
```
